// lscp_server.h

#ifndef __LSCP_SERVER_H
#define __LSCP_SERVER_H

#include "lscp_socket.h"

#if defined(__cplusplus)
extern "C" {
#endif

//-------------------------------------------------------------------------
// Server sockets.

struct _lscp_server_t;

typedef struct _lscp_connect_t
{
    struct _lscp_server_t  *server;
    lscp_socket_agent_t     client;
    int                     port;
    int                     ping;
    struct _lscp_connect_t *prev;
    struct _lscp_connect_t *next;
    
} lscp_connect_t;

typedef struct _lscp_connect_list_t
{
    lscp_connect_t *first;
    lscp_connect_t *last;
    unsigned int    count;
    lscp_mutex_t    mutex;

} lscp_connect_list_t;

typedef lscp_status_t (*lscp_server_proc_t)
(
    lscp_connect_t *pConnect,
    const char *pchBuffer,
    int cchBuffer,
    void *pvData
);

typedef struct _lscp_server_t {

    lscp_connect_list_t connects;
    lscp_server_proc_t  pfnCallback;
    void               *pvData;
    lscp_socket_agent_t tcp;
    lscp_socket_agent_t udp;
    lscp_thread_t      *pWatchdog;
    int                 iWatchdog;
    int                 iSleep;

} lscp_server_t;

lscp_server_t*  lscp_server_create      (int iPort, lscp_server_proc_t pfnCallback, void *pvData);
lscp_status_t   lscp_server_join        (lscp_server_t *pServer);
lscp_status_t   lscp_server_destroy     (lscp_server_t *pServer);

lscp_status_t   lscp_server_broadcast   (lscp_server_t *pServer, const char *pchBuffer, int cchBuffer);

lscp_status_t   lscp_server_result      (lscp_connect_t *pConnect, const char *pchBuffer, int cchBuffer);

lscp_status_t   lscp_server_subscribe   (lscp_connect_t *pConnect, int iPort);
lscp_status_t   lscp_server_unsubscribe (lscp_connect_t *pConnect, const char *pszSessionID);


#if defined(__cplusplus)
}
#endif

#endif // __LSCP_SERVER_H

// end of lscp_server.h
